/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 *
 * Terms of the MIT License:
 * ---------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of
 * the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package cd.casic.ci.process.process.dal.record;

import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.pojo.time.BuildTimestampType;
import cd.casic.ci.process.api.process.pojo.pipeline.record.BuildRecordContainer;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 构建记录容器 Mapper 接口
 */
@Mapper
@Repository
public interface BuildRecordContainerDao {

    /**
     * 批量保存构建记录容器
     *
     * @param records 构建记录容器列表
     */
    void batchSave(@Param("records") List<BuildRecordContainer> records);

    /**
     * 更新构建记录容器
     *
     * @param projectId    项目ID
     * @param pipelineId   流水线ID
     * @param buildId      构建ID
     * @param containerId  容器ID
     * @param executeCount 执行次数
     * @param containerVar 容器变量
     * @param buildStatus  构建状态
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @param timestamps   时间戳
     */
    void updateRecord(
            @Param("projectId") String projectId,
            @Param("pipelineId") String pipelineId,
            @Param("buildId") String buildId,
            @Param("containerId") String containerId,
            @Param("executeCount") Integer executeCount,
            @Param("containerVar") Map<String, Object> containerVar,
            @Param("buildStatus") BuildStatus buildStatus,
            @Param("startTime") LocalDateTime startTime,
            @Param("endTime") LocalDateTime endTime,
            @Param("timestamps") Map<BuildTimestampType, String> timestamps
    );

    /**
     * 重试时清空结束时间
     *
     * @param projectId    项目ID
     * @param pipelineId   流水线ID
     * @param buildId      构建ID
     * @param containerId  容器ID
     * @param executeCount 执行次数
     */
    void flushEndTimeWhenRetry(
            @Param("projectId") String projectId,
            @Param("pipelineId") String pipelineId,
            @Param("buildId") String buildId,
            @Param("containerId") String containerId,
            @Param("executeCount") Integer executeCount
    );

    /**
     * 获取构建记录容器
     *
     * @param projectId    项目ID
     * @param pipelineId   流水线ID
     * @param buildId      构建ID
     * @param containerId  容器ID
     * @param executeCount 执行次数
     * @return 构建记录容器
     */
    BuildRecordContainer getRecord(
            @Param("projectId") String projectId,
            @Param("pipelineId") String pipelineId,
            @Param("buildId") String buildId,
            @Param("containerId") String containerId,
            @Param("executeCount") Integer executeCount
    );

    /**
     * 获取构建记录容器列表
     *
     * @param projectId      项目ID
     * @param pipelineId     流水线ID
     * @param buildId        构建ID
     * @param executeCount   执行次数
     * @param stageId        阶段ID
     * @param matrixGroupId  矩阵组ID
     * @param buildStatusSet 构建状态集合
     * @return 构建记录容器列表
     */
    List<BuildRecordContainer> getRecords(
            @Param("projectId") String projectId,
            @Param("pipelineId") String pipelineId,
            @Param("buildId") String buildId,
            @Param("executeCount") Integer executeCount,
            @Param("stageId") String stageId,
            @Param("matrixGroupId") String matrixGroupId,
            @Param("buildStatusSet") Set<BuildStatus> buildStatusSet
    );

    /**
     * 更新构建记录容器状态
     *
     * @param projectId    项目ID
     * @param pipelineId   流水线ID
     * @param buildId      构建ID
     * @param executeCount 执行次数
     * @param buildStatus  构建状态
     * @param stageId      阶段ID
     */
    void updateRecordStatus(
            @Param("projectId") String projectId,
            @Param("pipelineId") String pipelineId,
            @Param("buildId") String buildId,
            @Param("executeCount") Integer executeCount,
            @Param("buildStatus") BuildStatus buildStatus,
            @Param("stageId") String stageId
    );

    /**
     * 获取最新的普通构建记录容器列表
     *
     * @param projectId    项目ID
     * @param pipelineId   流水线ID
     * @param buildId      构建ID
     * @param executeCount 执行次数
     * @param stageId      阶段ID
     * @return 构建记录容器列表
     */
    List<BuildRecordContainer> getLatestNormalRecords(
            @Param("projectId") String projectId,
            @Param("pipelineId") String pipelineId,
            @Param("buildId") String buildId,
            @Param("executeCount") Integer executeCount,
            @Param("stageId") String stageId
    );

    /**
     * 获取最新的矩阵构建记录容器列表
     *
     * @param projectId    项目ID
     * @param pipelineId   流水线ID
     * @param buildId      构建ID
     * @param executeCount 执行次数
     * @return 构建记录容器列表
     */
    List<BuildRecordContainer> getLatestMatrixRecords(
            @Param("projectId") String projectId,
            @Param("pipelineId") String pipelineId,
            @Param("buildId") String buildId,
            @Param("executeCount") Integer executeCount
    );
} 